@@ -1,9 +1,6 @@
 use crate::{
     database::{transaction::TransactionIndex, Database, KvStoreError},
-    model::{
-        coin::{Coin, CoinStatus},
-        fuel_block::{BlockHeight, FuelBlock, FuelBlockDb},
-    },
+    model::{BlockHeight, Coin, CoinStatus, FuelBlock, FuelBlockDb},
     service::Config,
     tx_pool::TransactionStatus,
 };
@@ -165,29 +162,24 @@ impl Executor {
 
             let status = if vm_result.should_revert() {
                 self.log_backtrace(&vm, vm_result.receipts());
-                // if script result exists, log reason
-                if let Some((script_result, _)) = vm_result.receipts().iter().find_map(|r| {
-                    if let Receipt::ScriptResult { result, gas_used } = r {
-                        Some((result, gas_used))
-                    } else {
-                        None
-                    }
-                }) {
-                    TransactionStatus::Failed {
-                        block_id: Default::default(),
-                        time: block.header.time,
-                        reason: format!("{:?}", script_result.reason()),
-                        result: Some(*vm_result.state()),
-                    }
-                }
-                // otherwise just log the revert arg
-                else {
-                    TransactionStatus::Failed {
-                        block_id: Default::default(),
-                        time: block.header.time,
-                        reason: format!("{:?}", vm_result.state()),
-                        result: Some(*vm_result.state()),
-                    }
+                // get reason for revert
+                let reason = vm_result
+                    .receipts()
+                    .iter()
+                    .find_map(|receipt| match receipt {
+                        // Format as `Revert($rA)`
+                        Receipt::Revert { ra, .. } => Some(format!("Revert({})", ra)),
+                        // Display PanicReason e.g. `OutOfGas`
+                        Receipt::Panic { reason, .. } => Some(format!("{}", reason.reason())),
+                        _ => None,
+                    })
+                    .unwrap_or_else(|| format!("{:?}", vm_result.state()));
+
+                TransactionStatus::Failed {
+                    block_id: Default::default(),
+                    time: block.header.time,
+                    reason,
+                    result: Some(*vm_result.state()),
                 }
             } else {
                 // else tx was a success
@@ -665,7 +657,7 @@ impl From<crate::state::Error> for Error {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::model::fuel_block::FuelBlockHeader;
+    use crate::model::FuelBlockHeader;
     use chrono::{TimeZone, Utc};
     use fuel_asm::Opcode;
     use fuel_crypto::SecretKey;
